from datetime import datetime
from typing import Optional, Sequence

from pydantic import BaseModel


class Person(BaseModel):
    personid: int
    plastname: str
    pfirstname: str
    psurname: str
    ppsex: int
    isuser: bool
    pilogin: str
    pipassword: str
    caid: str
    caidname: str
    rspostid: str
    rspostidname: str
    personemail: str
    pstatus: int
    pextcode: str


class Persons:
    def __init__(self, persons: Sequence[Person], count: int):
        self._persons = persons
        self.count = count

    def __getitem__(self, key: int) -> Person:
        return self._persons[key]


class Measure(BaseModel):
    meid: int
    mename: str
    medescription: str
    metype: str
    mecode: str
    mestatus: str
    mestartdate: Optional[datetime] = None
    meenddate: Optional[datetime] = None
    meeduform: Optional[int] = None
    mecontenttype: Optional[int] = None
    testid: Optional[str] = None
    testidname: Optional[str] = None
    ugrid: Optional[int] = None
    ugridname: Optional[str] = None
    mepasses: Optional[int] = None

    def __str__(self):
        result = f"{self.mename}\n"
        if self.mestartdate:
            result += f"Начало: {self.mestartdate.strftime('%d.%m.%Y')}\n"
        if self.meenddate:
            result += f"Окончание: {self.meenddate.strftime('%d.%m.%Y')}\n"
        return result


class Measures:
    def __init__(self, measures: Sequence[Measure], count: int):
        self._measures = measures
        self.count = count

    def __getitem__(self, key: int) -> Measure:
        return self._measures[key]



from typing import Optional

from .base_api import BaseAPI
from .models import Measure, Person, Measures, Persons


class VirtualRoom(BaseAPI):
    def __init__(
            self,
            base_link: str,
            secret_key: str,
            app_id: str
    ):
        super().__init__(
            base_link,
            secret_key,
            app_id
        )

    async def get_persons(self, limit: int = 200, offset: int = 0) -> Optional[Persons]:
        """
        Получение информации о физических лицах
        :rtype: list[Person]
        :param limit: Количество записей (200 максимально)
        :param offset: Сдвиг страницы
        :return: List of Persons
        """
        persons = await self._get_json(
            route="/service/v2/persons",
            params={
                "limit": limit,
                "offset": offset,
            }
        )
        if persons:
            return Persons(
                [Person(**person) for person in persons['data']],
                persons['count']
            )
        else:
            return None

    async def get_person(self, person_id: int) -> Optional[Person]:
        """
        Получение информации о физическом лице
        :rtype: Person
        :param person_id: идентификатор физического лица
        :return: Person
        """
        person = await self._get_json(
            route=f"/service/v2/persons/{person_id}"
        )
        if person:
            return Person(**person)
        else:
            return None

    async def get_measures(self, limit: int = 200, offset: int = 0) -> Optional[Measures]:
        """
        Получение информации о мероприятиях
        :rtype: list[Measure]
        :param limit: Количество записей (200 максимально)
        :param offset: Сдвиг страницы
        :return: List of Measures
        """
        measures = await self._get_json(
            route="/service/v2/measures",
            params={
                "limit": limit,
                "offset": offset,
            }
        )
        if measures:
            return Measures(
                [Measure(**measure) for measure in measures['data']],
                measures['count']
            )
        else:
            return None

    async def get_measure(self, measure_id: int) -> Measure:
        """
        Получение информации о мероприятии
        :rtype: Measure
        :param measure_id: идентификатор мероприятия
        :return: Measure
        """
        measure = await self._get_json(
            route=f"/service/v2/measures/{measure_id}"
        )
        if measure:
            return Measure(**measure)

    async def get_measures_info(self):
        measures_info = await self._get_json(
            route="/service/v2/measures/info"
        )
        return measures_info
